// Copyright 2007 Messiah College. All rights reserved.
// Jason Long <jlong@messiah.edu>
//
// autocomplete and related changes
// Copyright 2004 Leslie A. Hensley
// hensleyl@papermountain.org
// you have a license to do what ever you like with this code
// orginally from Avai Bryant 
// http://www.cincomsmalltalk.com/userblogs/avi/blogView?entry=3268075684

//
// do some browser detection when the page loads
//

if (navigator.userAgent.indexOf("Safari") > 0)
{
  isSafari = true;
  isMoz = false;
  isIE = false;
}
else if (navigator.product == "Gecko")
{
  isSafari = false;
  isMoz = true;
  isIE = false;
}
else
{
  isSafari = false;
  isMoz = false;
  isIE = true;
}
    
function recreateTR(parentElement, subtree)
{
  for(var i = parentElement.childNodes.length-1; i>=0; i--)
  {
    parentElement.removeChild(parentElement.childNodes[i]);
  }

  for(var i=0; i<subtree.childNodes.length; i++)
  {
    var cell = document.createElement(subtree.childNodes[i].nodeName);
    cell.innerHTML = flattenChildren(subtree.childNodes[i].childNodes)
    parentElement.appendChild(cell);
  }
}

/**
 * Generates a no-argument function that can be called whenever
 * the value of the input box changes.
 * @param uriFunc function that determines what URL to query
 * @param postFunc function to call after results are returned
 * @param preFunc function to call before request is initiated
 * @param popupId id of the element to be updated with the results
 */
function liveUpdater(uriFunc, postFunc, preFunc, popupId)
{
	if(!postFunc) postFunc = function () {};
	if(!preFunc) preFunc = function () {};

	var request = false;
	if (window.XMLHttpRequest)
	{
		// in Mozilla, we create a new XMLHttpRequest once per input box
		request = new XMLHttpRequest();
	}
    
    function update()
    {
        if(request && request.readyState < 4)
            request.abort();

	// in IE, we create a new XMLHttpRequest object for each query
        if(!window.XMLHttpRequest)
            request = new ActiveXObject("Microsoft.XMLHTTP");

        preFunc();
        request.onreadystatechange = processRequestChange;
        request.open("GET", uriFunc());
        request.send(null);
        return true;
    }

    function processRequestChange()
    {
      if(request.readyState == 4 && request.status == 200)
      {
        var xmlDoc = request.responseXML

        var body = xmlDoc.getElementsByTagName("body");

        if(body.length>0)
        {
          var nodes = body[0].childNodes
          for(var i=0;i<nodes.length;i++)
          {
            if(nodes[i].nodeType==1 && nodes[i].getAttribute("id")!=null)
            {
              var id = nodes[i].getAttribute("id");
	      if (id == "popup" && popupId != null) id = popupId;

              if(isIE && nodes[i].nodeName == 'tr')
              {
                recreateTR(document.getElementById(id), nodes[i]);
              }
              else
              {
				var tmp = flattenChildren(nodes[i].childNodes);
				//alert("tmp = " + tmp);
				//alert("innerHTML = " + nodes[i].innerHTML);
		var docElement = document.getElementById(id);
		if (docElement != null)
			docElement.innerHTML = flattenChildren(nodes[i].childNodes)
		else
			alert("element id=" + id + " not found in document");
              }
            }
          }
        }

        var scripts = xmlDoc.getElementsByTagName("script");
        for(var i=0;i<scripts.length;i++)
        {
          if(scripts[i].firstChild!=null)
          {
            var script = scripts[i].firstChild.nodeValue
            if(script != null)
            {
              eval(script)
            }
          }
        }
        postFunc();
      }
      else if (request.readyState == 4)
      {
      	//alert("readyState=" + request.readyState + ", status="
		//+ request.status);
      }
    }

    return update;
}

function autocomplete(id, popupId, uri)
{
	var inputField = document.getElementById(id);
	var popup = document.getElementById(popupId);
	var options = new Array(); 
	var current = 0;
	var originalPopupTop = popup.offsetTop; 
	var hourglassImage = document.getElementById(id + "-hourglass");

	function setHourglass(newVisibility)
	{
		if (hourglassImage != null)
		{
			hourglassImage.style.visibility = newVisibility;
		}
	}

    function constructUri()
    {
		setHourglass('visible');
        var separator = "?";
        if(uri.indexOf("?") >= 0)
            separator = "&";
        var queryUrl = uri + separator + "s=" + escape(inputField.value);
	//alert("url = " + queryUrl);
        return queryUrl;
    }
   
    function hidePopup()
    {
	  setHourglass('hidden');
      popup.style.visibility = 'hidden';
    }

    function handlePopupOver()
    {
      removeListener(inputField, 'blur', hidePopup);
    }
    
    function handlePopupOut()
    {
      if(popup.style.visibility == 'visible')
      {
        addListener(inputField, 'blur', hidePopup);
      }
    }
    
    function setInputValue(optionElement)
    {
      var newValue = optionElement.getAttribute("value");
      if (newValue == null)
      {
        newValue = optionElement.innerHTML;
      }
	  //alert("setting input field to '" + newValue + "'");
      inputField.value = newValue;
    }

    function handleClick(e)
    {
      setInputValue(eventElement(e));
      popup.style.visibility = 'hidden';
      inputField.focus();
    }
    
    function handleOver(e)
    {
      options[current].className = '';
      current = eventElement(e).index;
      options[current].className = 'selected';
    }

	// called when autocompletion-info is received
	function post()
	{
		current = 0;
		options = popup.getElementsByTagName("li");
		if((options.length > 1)
           || (options.length == 1 
               && options[0].innerHTML != inputField.value))
		{
          setPopupStyles();
          for(var i = 0; i < options.length; i++)
          {
            options[i].index = i;
            addOptionHandlers(options[i]);
          }
          options[0].className = 'selected';
		}
		else
		{
          popup.style.visibility = 'hidden';
		}
		setHourglass('hidden');
	}
  
    function setPopupStyles()
    {
      var maxHeight
      if(isIE)
      {
        maxHeight = 200;
        //popup.style.left = '0px';
        //popup.style.top = (originalPopupTop + inputField.offsetHeight) + 'px';
      }
      else
      {
        maxHeight = window.outerHeight/3;
      }
      if(popup.offsetHeight < maxHeight)
      {
        popup.style.overflow = 'hidden';
      }
      else if(isMoz)
      {
        popup.style.maxHeight = maxHeight + 'px';
        popup.style.overflow = '-moz-scrollbars-vertical';
      }
      else
      {
        popup.style.height = maxHeight + 'px';
        popup.style.overflowY = 'auto';
      }
      popup.scrollTop = 0;
      popup.style.visibility = 'visible';
    }
    
    function addOptionHandlers(option)
    {
      addListener(option, "click", handleClick);
      addListener(option, "mouseover", handleOver);
    }
    
    var updater = liveUpdater(constructUri, post, null, popupId);
    var timeout = false;
   
	// called when a key event occurs
    function start(e) {
      if (timeout)
        window.clearTimeout(timeout);
      //up arrow
      if(e.keyCode == 38)
      {
        if(current > 0)
        {
          options[current].className = '';
          current--;
          options[current].className = 'selected';
          options[current].scrollIntoView(false);
        }
      }
      //down arrow
      else if(e.keyCode == 40)
      {
        if(current < options.length - 1)
        {
          options[current].className = '';
          current++;
          options[current].className = 'selected';
          options[current].scrollIntoView(false);
        }
      }
      //enter or tab
      else if((e.keyCode == 13 || e.keyCode == 9) && popup.style.visibility == 'visible')
      {
        setInputValue(options[current]);
		hidePopup();
        inputField.focus();
        if(isIE)
        {
          event.returnValue = false;
        }
        else
        {
          e.preventDefault();
        }
      }
      else
      {
        timeout = window.setTimeout(updater, 300);
      }
    }
  addKeyListener(inputField, start);
  addListener(popup, 'mouseover', handlePopupOver);
  addListener(popup, 'mouseout', handlePopupOut);
}

/* Functions to handle browser incompatibilites */
function eventElement(event)
{
  if(isMoz)
  {
    return event.currentTarget;
  }
  else
  {
    return event.srcElement;
  }
}

function addKeyListener(element, listener)
{
  if (isSafari)
    element.addEventListener("keydown",listener,false);
  else if (isMoz)
    element.addEventListener("keypress",listener,false);
  else
    element.attachEvent("onkeydown",listener);
}

function addListener(element, type, listener)
{
  if(element.addEventListener)
  {
    element.addEventListener(type, listener, false);
  }
  else
  {
    element.attachEvent('on' + type, listener);
  }
}

function removeListener(element, type, listener)
{
  if(element.removeEventListener)
  {
    element.removeEventListener(type, listener, false);
  }
  else
  {
    element.detachEvent('on' + type, listener);
  }
}

/* XML Helper functions */
function flatten(node)
{
	if(node.nodeType == 1)
	{
		return '<' + node.nodeName + flattenAttributes(node) + '>' +
		flattenChildren(node.childNodes) + '</' + node.nodeName + '>';
	}
	else if(node.nodeType == 3)
	{
		return node.nodeValue;
	}
}

function flattenAttributes(node)
{
  var buffer = ''
  for(var i=0;i<node.attributes.length;i++)
  {
    var attribute = node.attributes[i]
    buffer += ' '+attribute.name+'="'+attribute.value+'"'
  }
  return buffer;
}

function flattenChildren(nodes)
{
	var buffer = '';
	if(nodes.length > 0)
	{
		for (var i=0;i<nodes.length;i++)
		{
			buffer += flatten(nodes[i]);
		}
	}
	return buffer;
}
